# Running VOTpy

## Installation

Clone the [vot-toolkit](https://github.com/votchallenge/toolkit) from github for example to `$HOME/github/toolkit`.

Open the file `$HOME/github/toolkit/vot/utilities/cli.py` and add the lines

```
if __name__ == '__main__':
    main()
```
at the end of the file.

Then, navigate to `$HOME/github/toolkit` and run

```python -m vot.utilities.cli initialize vot2022/stb --workspace $SCRATCH/vot/vot2022-stb```

replace the workspace path with any path you like but make sure you have enough disk space there.
The tracking sequences will be downloaded to this folder.

This will initialize a vot workspace at `$SCRATCH/vot/vot2022-stb` for the specified stack, in this case `vot2022/stb`.
Other available stacks are listed [here](https://github.com/votchallenge/toolkit/tree/master/vot/stack). 
Alternatively you can inspect them at `$HOME/github/toolkit/vot/stack` you can also add your own if you want to run only a subset of the specified experiments.

Besides setting up the workspace it will also download the tracking sequences taking a few GB of space depening on the stack,
so make sure that you have enough disk space available.

If you run into `OS.Errors` stating that you don't have enought disk space, this could also mean that your `$TMP` folder
is running out of space because the sequences are first stored there and then moved to the workspace.

Once the workspace is initialized and the sequences are downloaded, please add the following variables to the `pytracking/evaluation/local.py` file:

```
settings.vot_ws_path = '$SCRATCH/vot/vot2022-stb'
settings.vot2022_path = '$SCRATCH/vot/vot2022-stb/sequences'
settings.vot_toolkit_path = '$HOME/github/toolkit'
```

## Running a single Sequence

The vot2022 sequences can then be used for debugging purposes as other datasets (OTB, LaSOT etc.) via the files
`pytracking/evaluation/vot2022dataset.py` by running 

```
pytracking.run_tracker($tracker_name, $tracker_params,
                       dataset_name='vot2022', sequence='conduction1', debug=3,
                       visdom_info={'use_visdom': True, 'server': "129.132.67.103", 'port': 8097})
``` 

currently only these two datasets are supported using the VOTpy toolkit for visualization and debugging purposes. 
However, Evaluation and Analysis should be supported for any available stack.

## Evaluating and Analysing a full VOT Challenge.

Please create an experiment file in `pytracking/VOTpy/experiments/$EXP_FILE_NAME.py` such as 
```
from pytracking.evaluation import trackerlist

def $EXP_NAME():
    trackers = trackerlist($tracker_name, $tracker_params, 0)
    return trackers
```

that specifies the tracker to be run.

Navigate to `pytracking/VOTpy` and run `python vot_evaluate.py $EXP_FILE_NAME $EXP_NAME`.
The workspace and stack are read from the settings.
This should evaluate the tracker on all sequences using the stack specific evaluation procedure.
The raw results will be stored at `$SCRATCH/vot/vot2022-stb/results`.

In case of a failure check the reported log files or try to run again. 

Once all sequences are evaluated run `python vot_analysis.py $EXP_FILE_NAME $EXP_NAME` and the
results will be available at `$SCRATCH/vot/vot2022-stb/analysis`. By default the results will be saved in `html` format
this can be changed to for example `json` by changing the '.format' attribute in `class Config` in `pytracking/VOTpy/vot_analysis.py`.  

If you want to display different experiments in the same `html` file just create a new experiment in the experiment file with
the already evaluated trackers and specify this experiment when running `vot_analysis.py'.`

## Packing

Run `python vot_pack.py $EXP_FILE_NAME $EXP_NAME` in order to pack the results to submit it to the VOT challenge when participating.
