import os
import subprocess
from pytracking.evaluation.environment import env_settings

COPY_CMDS = ['rsync', '-a', '--include', '*/', '--exclude', '*.tar', '{}', '{}']
LIST_CMDS = ['find', '{}', '-name', '*.tar', '-exec', 'realpath', '--relative-to', '{}', '{}', ';']
EXT_CMDS = ['parallel', '--will-cite', '-j', '8', 'tar', 'xf' ,'{}/{{}}', '-C', '{}/{{//}}']


def prepare_test_data_euler(name):
    dataset_dict = dict(
        nfs=env_settings().nfs_root_euler,
        otb=env_settings().otb_root_euler,
        uav=env_settings().uav_root_euler,
        lasot=env_settings().lasot_root_euler,
        lasot_extension_subset=env_settings().lasot_extension_subset_root_euler,
        oxuva_dev=env_settings().oxuva_root_euler,
        oxuva_test=env_settings().oxuva_root_euler
    )

    prepare_data_euler(src_data_dir=dataset_dict[name])


def prepare_data_euler(src_data_dir):
    tmp_dir = os.getenv('TMPDIR', '/tmp')
    dataset_folder_name = os.path.basename(src_data_dir)

    dest_data_dir = os.path.join(tmp_dir, dataset_folder_name)

    marker_path = os.path.join(dest_data_dir, 'marker.txt')

    print('Start copying dataset {name} from {src} to {dest}'.format(name=dataset_folder_name,
                                                                     src=src_data_dir, dest=dest_data_dir))

    if not os.path.exists(marker_path):
        copy_cmds = list(COPY_CMDS)
        copy_cmds[6] = copy_cmds[6].format(src_data_dir)
        copy_cmds[7] = copy_cmds[7].format(tmp_dir)

        # copy all subdirectories and file except tar files from src_data_dir to dest_data_dir.
        subprocess.call(copy_cmds)

        # untaring the dataset
        list_cmds = list(LIST_CMDS)
        list_cmds[1] = list_cmds[1].format(src_data_dir)
        list_cmds[7] = list_cmds[7].format(src_data_dir)

        ext_cmds = list(EXT_CMDS)
        ext_cmds[6] = ext_cmds[6].format(src_data_dir)
        ext_cmds[8] = ext_cmds[8].format(dest_data_dir)

        ps = subprocess.Popen(list_cmds, stdout=subprocess.PIPE)
        subprocess.check_output(ext_cmds, stdin=ps.stdout)
        ps.wait()

        with open(marker_path, 'w') as f:
            pass

    print('Finished copying data')

