import subprocess
import json


def main():
    out = subprocess.check_output(['bjobs', '-o', 'jobid: job_name: jobindex: stat: start_time', '-json'])

    d = json.loads(out.decode('utf-8'))

    jobs = d['RECORDS']
    jobs_sorted = sorted(jobs, key=lambda v: (v['JOBID'], v['JOBINDEX']))

    print('{done: >4} {tot: >4} {stat: >6} {time: >16} {jobid: >10} {name}'.format(done='DONE', tot='TOT', stat='STAT',
                                                                                   time='START TIME', jobid='JOBID',
                                                                                   name='JOB_NAME'))

    for job in jobs_sorted:
        try:
            # skip non array jobs
            if int(job['JOBINDEX']) == 0:
                continue

            if job['STAT'] != 'RUN':
                vals = dict(done=0, tot=0, stat=job['STAT'], time='Not Started', jobid=job['JOBID'], name=job['JOB_NAME'])
                print('{done: >4} {tot: >4} {stat: >6} {time: >16} {jobid: >10} {name}'.format(**vals))
                continue

            out = subprocess.check_output(['bpeek', '{}[{}]'.format(job['JOBID'], job['JOBINDEX'])])
            lines = out.decode('utf-8').splitlines()

            title_lines = [[l for l in line.split(' ') if l != '']
                           for line in lines if 'Evaluating' in line and 'trackers on' in line]

            # If job has started but not logged anything useful yet.
            if len(title_lines) == 0:
                vals = dict(done=0, tot=0, stat=job['STAT'], time=job['START_TIME'], jobid=job['JOBID'], name=job['JOB_NAME'])
                print('{done: >4} {tot: >4} {stat: >6} {time: >16} {jobid: >10} {name}'.format(**vals))
                continue

            title = title_lines[0]
            num_seq = int(title[4])
            num_trackers = int(title[1])

            num_seq_tot = num_seq * num_trackers

            num_seq_done = sum([1 for l in lines if 'FPS' in l])
            vals = dict(done=num_seq_done, tot=num_seq_tot, stat=job['STAT'], time=job['START_TIME'],
                        jobid=job['JOBID'], name=job['JOB_NAME'])
            print('{done: >4} {tot: >4} {stat: >6} {time: >16} {jobid: >10} {name}'.format(**vals))
        except:
            pass


if __name__ == '__main__':
    main()
