import os
import json
import numpy as np

from tqdm import tqdm

from pytracking.evaluation import Tracker
from pytracking.evaluation.environment import env_settings


def save_bb(file, data):
    tracked_bb = np.array(data).astype(int)
    np.savetxt(file, tracked_bb, delimiter='\t', fmt='%d')


def main(tracker, dataset_name):
    if tracker.run_id is None:
        results_path_json = '{}/{}_{}.json'.format(tracker.results_dir, tracker.parameter_name, dataset_name)
    else:
        results_path_json = '{}/{}_{:0>3}_{}.json'.format(tracker.results_dir, tracker.parameter_name, tracker.run_id,
                                                          dataset_name)

    if os.path.isfile(results_path_json):
        with open(results_path_json, 'r') as f:
            d = json.load(f)
    else:
        raise FileNotFoundError(results_path_json)

    # root_path = os.path.join(env_settings().results_path, '{}_extracted_results'.format(tracker.name), tracker.parameter_name)
    root_path = os.path.join(env_settings().results_path, 'tomp_ext', 'tomp50_cvpr')

    if not os.path.exists(root_path):
        os.makedirs(root_path)

    for name, data in tqdm(d.items()):
        txt_path = os.path.join(root_path, "{}.txt".format(name))
        save_bb(txt_path, data)


if __name__ == '__main__':
    tracker = Tracker('euler_sync_dir_sst',
                      'sst_label_and_ltrb_box_enc_norm_cntr_smpl_18_300_no_background_label_encoding_two_stage_parallel_org_mem_090_rescale',
                      # 'sst_label_and_ltrb_box_enc_norm_cntr_smpl_300_resnet101_no_background_label_encoding_two_stage_parallel_org_mem_090_rescale',
                      run_id=0)
    main(tracker, 'uav')